@@ -7,11 +7,13 @@ use std::fs;
 use crate::arena::Options;
 use crate::arena::ArenaTree;
 use crate::arena::Config;
+use crate::arena::PIPEBLOCK;
 use crate::arena::{newick2tree,xml2tree};
 use crate::arena::{knuth_layout,cladogramme,check_contour_postorder,shift_mod_xy,
-    set_middle_postorder,real_length,set_leaves_y_values};
+    set_middle_postorder,real_length,set_leaves_y_values,shift_nodes_y_values};
 use crate::arena::{map_species_trees,set_species_width,check_vertical_contour_postorder,
-    bilan_mappings,center_gene_nodes,move_dupli_mappings,move_species_mappings,move_species_mappings_fl};
+    bilan_mappings,center_gene_nodes,move_dupli_mappings,move_species_mappings,
+    species_uniformisation,process_fl};
 use crate::arena::{find_sptrees,find_rgtrees,check_for_obsolete,scale_heigth,scale_width};
 use crate::thirdlevel::{get_gtransfer,optimisation,check_optimisation,classify_transfer,reorder_transfers};
 use crate::drawing::{draw_tree,draw_sptree_gntrees};
@@ -23,7 +25,7 @@ pub fn read_newick(filename:String, tree: &mut ArenaTree<String>) {
     let contents = match contents {
         Ok(contents) => contents,
         Err(err) => {
-            eprintln!("Error! Something went wrong reading the newick file.");
+            eprintln!("\nERROR:\nSomething went wrong when reading the newick file.");
             eprintln!("{}",err);
             eprintln!("Please check file name and path.");
             std::process::exit(1);
@@ -40,13 +42,23 @@ pub fn read_phyloxml(filename:String, tree: &mut ArenaTree<String>) {
     let contents = match contents {
         Ok(contents) => contents,
         Err(err) => {
-            eprintln!("Error! Something went wrong reading the phyloxml file.");
+            eprintln!("\nERROR:\nSomething went wrong when reading the phyloxml file.");
             eprintln!("{}",err);
             eprintln!("Please check file name and path.");
             std::process::exit(1);
         },
     };
-    let doc = roxmltree::Document::parse(&contents).unwrap();
+
+    let doc = roxmltree::Document::parse(&contents);
+    let doc = match doc {
+        Ok(xml) => xml,
+        Err(_err) => {
+            eprintln!("\nERROR:\nSomething went wrong when parsing the input file.");
+            eprintln!("It seems this is not a xml file");
+            std::process::exit(1);
+        },
+    };
+
     let descendants = doc.root().descendants();
     // Search for the first occurence of the "clade" tag
     for node in descendants {
@@ -227,13 +239,22 @@ pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<Strin
     let contents = match contents {
         Ok(contents) => contents,
         Err(err) => {
-            eprintln!("Error! Something went wrong reading the recPhyloXML file.");
+            eprintln!("\nERROR:\nSomething went wrong when reading the recPhyloXML file.");
             eprintln!("{}",err);
             eprintln!("Please check file name and path.");
             std::process::exit(1);
         },
     };
-    let doc = &mut roxmltree::Document::parse(&contents).unwrap();
+    // let doc = &mut roxmltree::Document::parse(&contents).unwrap();
+    let doc = roxmltree::Document::parse(&contents);
+    let doc = &mut match doc {
+        Ok(contents) => contents,
+        Err(_err) => {
+            eprintln!("\nERROR:\nSomething went wrong when parsing the input file.");
+            eprintln!("It seems this is not a xml file");
+            std::process::exit(1);
+        },
+    };
     // Get the species trees:
     // Get the list of nodes associated to  the "spTree" tag
     // let spnodes = find_sptrees(doc).expect("No clade spTree has been found in xml");
@@ -241,7 +262,7 @@ pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<Strin
     let spnodes = match spnodes {
         Ok(indexes) => indexes,
         Err(_err) => {
-            eprintln!("\nError: No clade spTree has been found in xml.\
+            eprintln!("\nERROR:\nNo clade spTree has been found in xml.\
             \nIt seems that the input file is not a recPhyloXML file.\
             \nUse option -F to force to use phyloXML or newick format.");
              std::process::exit(1);
@@ -288,7 +309,7 @@ pub fn read_recphyloxml_multi(filename:String, global_pipe: &mut ArenaTree<Strin
     let rgnodes = match rgnodes {
         Ok(indexes) => indexes,
         Err(_err) => {
-            eprintln!("\nError: No clade recGeneTree has been found in xml.\
+            eprintln!("\nERROR:\nNo clade recGeneTree has been found in xml.\
             \nIt seems that the input file is not a recPhyloXML file.\
             \nUse option -F to force to use phyloXML or newick format.");
              std::process::exit(1);
@@ -445,6 +466,37 @@ if mapping {
     map_species_trees(&mut sp_tree,&mut gene_trees);
     info!("Species tree after mapping : {:?}",sp_tree);
 }
+
+//  Option: uniformise les noeuds de l'arbre d'espece
+if options.uniform {
+    species_uniformisation(&mut sp_tree);
+    // Il fait decaler le tout (mais je ne comprend pas pourquoi?)
+    let smallest_y = sp_tree.get_smallest_y();
+    let root_width = sp_tree.arena[root].nbg as f32 * PIPEBLOCK ;
+    shift_nodes_y_values(sp_tree,root,-smallest_y + root_width);
+}
+// ---------------------------------------------------------
+// Option : utilise les longueurs de branches
+// ---------------------------------------------------------
+if options.real_length_flag {
+    //  Tout d'abord il faut uniformiser
+    species_uniformisation(&mut sp_tree);
+    let min_dist = sp_tree.get_smallest_l();
+    if min_dist == 0.0 {
+        eprintln!("\nERROR:\nFound a branch with a distance equal to 0.0 in the 'pipe' tree.");
+        eprintln!("It is not possible create a pipe tree presenting branches of zero length.");
+        std::process::exit(1);
+    }
+    // On veut que la longueur minimum soit un peu superieure a la moitie de l'epaisser des noods
+    // On utilise le nombre de gene , qui a etet uniformise precedemment
+    options.scale = (sp_tree.arena[root].nbg as f32 / 2.0  + 0.25)  / min_dist ;
+    real_length(&mut sp_tree, root, &mut 0.0, & options);
+    // Il fait decaler le tout (mais je ne comprend pas pourquoi?)
+    let smallest_y = sp_tree.get_smallest_y();
+    let root_width = sp_tree.arena[root].nbg as f32 * PIPEBLOCK ;
+    shift_nodes_y_values(sp_tree,root,-smallest_y + root_width);
+
+}
 // ---------------------------------------------------------
 // OPTIONAL Optimisation if needed
 // ---------------------------------------------------------
@@ -478,6 +530,7 @@ if options.optimisation {
         check_optimisation(&selected_transfers[numt], &mut sp_tree, tr_root);
     }
 }
+
 // ---------------------------------------------------------
 // 3eme étape : Vérifie les conflits dans l'arbre d'espèces
 // au niveau horizontal -> valeurs xmod
@@ -497,14 +550,23 @@ set_middle_postorder(&mut sp_tree, root);
 // 6ème etape : Fixe l'épaisseur de l'arbre d'espèces
 // ---------------------------------------------------------
 set_species_width(&mut sp_tree, &mut gene_trees);
+
 // ---------------------------------------------------------
 // 7ème étape :  Vérifie les conflits verticaux dans
 // l'arbre d'espèces
 // ---------------------------------------------------------
-check_vertical_contour_postorder(&mut sp_tree, root, 0.0);
+if ! options.real_length_flag {
+    check_vertical_contour_postorder(&mut sp_tree, root, 0.0);
+}
+// ---------------------------------------------------------
 // Egalise les feuilles
-let largest_y  = sp_tree.get_largest_y();
-set_leaves_y_values(sp_tree,root,largest_y);
+// ---------------------------------------------------------
+if ! options.real_length_flag {
+    // Egalise les feuilles
+    let largest_y  = sp_tree.get_largest_y();
+    set_leaves_y_values(sp_tree,root,largest_y);
+
+}
 // ---------------------------------------------------------
 // OPTIONAL Scale the heigt if needed
 // ---------------------------------------------------------
@@ -528,7 +590,7 @@ if options.width != 1.0 { scale_width(&mut sp_tree,options.width)};
 // 9ème etape : décale les noeuds de gene associés à un
 // noeud d'especes pour éviter qu'ils soit superposés
 // ---------------------------------------------------------
-bilan_mappings(&mut sp_tree, &mut gene_trees,root, & options);
+bilan_mappings(&mut sp_tree, &mut gene_trees,initial_root, & options);
 // ---------------------------------------------------------
 // 10ème étape : recalcule les coordonnées svg de tous les
 // arbres de gènes
@@ -562,11 +624,10 @@ move_dupli_mappings(&mut sp_tree, &mut gene_trees,initial_root);
 // ---------------------------------------------------------
 move_species_mappings(&mut sp_tree, &mut gene_trees,initial_root);
 
-if options.free_living {
-    center_gene_nodes(&mut sp_tree,&mut gene_trees,free_root);
-    move_dupli_mappings(&mut sp_tree, &mut gene_trees,free_root);
-    move_species_mappings_fl(&mut sp_tree, &mut gene_trees,free_root);
+// Option : traitement specifique des gene "free living"
 
+if options.free_living {
+    process_fl(&mut sp_tree, &mut gene_trees,free_root);
 }
 // // ---------------------------------------------------------
 // // Fin: Ecriture du fichier svg
